package pps.core.base.service;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import pps.core.base.entity.*;
import pps.core.base.mapper.*;
import pps.core.base.service.data.base_power_line.*;
import pps.core.base.service.data.base_power_line_diesel.CreateBasePowerLineDieselInput;
import pps.core.base.service.data.base_power_line_plant.CreateBasePowerLinePlantInput;
import pps.core.base.service.data.base_power_line_storage.CreateBasePowerLineStorageInput;
import pps.core.base.service.data.base_power_line_wellhead.CreateBasePowerLineWellheadInput;
import pps.core.base.utils.BaseEntUtils;
import pps.core.common.session.PpsUserSession;
import xstartup.annotation.XService;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.base.util.XCopyUtils;
import xstartup.data.XPageResult;
import xstartup.data.XServiceResult;
import xstartup.data.XSingleResult;
import xstartup.error.XError;
import xstartup.feature.api.annotation.XApiAnonymous;
import xstartup.feature.api.annotation.XApiGet;
import xstartup.feature.api.annotation.XApiPost;
import xstartup.helper.XTransactionHelper;

import java.util.List;

/**
 * 输电线路配置模块
 *
 * @author ZWT
 * @date 2023/08/25
 */
@XService
@XText("输电线路配置模块")
public class BasePowerLineService {

    /**
     * 输电线路配置--新增
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XServiceResult}
     */
    @XApiAnonymous
    @XApiPost
    @XText("输电线路配置--新增")
    public XServiceResult createBasePowerLine(XContext context, CreateBasePowerLineInput input) {
        //        PpsUserSession session = context.getSession(PpsUserSession.class);
        PpsUserSession session = new PpsUserSession();
        session.setId("123");
        session.setUserName("ceshi");
        return XTransactionHelper.begin(context, () -> {
            BasePowerLineMapper mapper = context.getBean(BasePowerLineMapper.class);
            BasePowerLineEnt entity = XCopyUtils.copyNewObject(input, BasePowerLineEnt.class);
            //新增输电线路
            BaseEntUtils.setBaseEntDefault(entity, session);
            mapper.insert(entity);
            String lineId = entity.getId();
            String ouId = entity.getOuId();
            //新增井口配置
            List<CreateBasePowerLineWellheadInput> wellheadInputs = input.getWellheadInputs();
            if (!wellheadInputs.isEmpty()) {
                List<BasePowerLineWellheadEnt> wellheads = XCopyUtils.copyNewList(wellheadInputs, BasePowerLineWellheadEnt.class);
                for (BasePowerLineWellheadEnt wellhead : wellheads) {
                    wellhead.setLineId(lineId);
                    wellhead.setOuId(ouId);
                    BaseEntUtils.setBaseEntDefault(wellhead, session);
                }
                BaseEntUtils.batchInsert(BasePowerLineWellheadMapper.class, BasePowerLineWellheadEnt.class, wellheads);
            }
            //新增光伏配置
            List<CreateBasePowerLinePlantInput> plantInputs = input.getPlantInputs();
            if (!plantInputs.isEmpty()) {
                List<BasePowerLinePlantEnt> plants = XCopyUtils.copyNewList(plantInputs, BasePowerLinePlantEnt.class);
                for (BasePowerLinePlantEnt plant : plants) {
                    plant.setLineId(lineId);
                    plant.setOuId(ouId);
                    BaseEntUtils.setBaseEntDefault(plant, session);
                }
                BaseEntUtils.batchInsert(BasePowerLinePlantMapper.class, BasePowerLinePlantEnt.class, plants);
            }
            //新增储能配置
            List<CreateBasePowerLineStorageInput> storageInputs = input.getStorageInputs();
            if (!storageInputs.isEmpty()) {
                List<BasePowerLineStorageEnt> storages = XCopyUtils.copyNewList(storageInputs, BasePowerLineStorageEnt.class);
                for (BasePowerLineStorageEnt storage : storages) {
                    storage.setLineId(lineId);
                    storage.setOuId(ouId);
                    BaseEntUtils.setBaseEntDefault(storage, session);
                }
                BaseEntUtils.batchInsert(BasePowerLineStorageMapper.class, BasePowerLineStorageEnt.class, storages);
            }
            //新增柴发配置
            List<CreateBasePowerLineDieselInput> dieselInputs = input.getDieselInputs();
            if (!dieselInputs.isEmpty()) {
                List<BasePowerLineDieselEnt> diesels = XCopyUtils.copyNewList(dieselInputs, BasePowerLineDieselEnt.class);
                for (BasePowerLineDieselEnt diesel : diesels) {
                    diesel.setLineId(lineId);
                    diesel.setOuId(ouId);
                    BaseEntUtils.setBaseEntDefault(diesel, session);
                }
                BaseEntUtils.batchInsert(BasePowerLineDieselMapper.class, BasePowerLineDieselEnt.class, diesels);
            }
            return XServiceResult.OK;
        });
    }

    /**
     * 输电线路配置--修改
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XServiceResult}
     */
    @XApiAnonymous
    @XApiPost
    public XServiceResult updateBasePowerLine(XContext context, UpdateBasePowerLineInput input) {
        //        PpsUserSession session = context.getSession(PpsUserSession.class);
        PpsUserSession session = new PpsUserSession();
        session.setId("123");
        session.setUserName("ceshi");
        return XTransactionHelper.begin(context, () -> {
            BasePowerLineMapper mapper = context.getBean(BasePowerLineMapper.class);
            QueryWrapper<BasePowerLineEnt> queryWrapper = new QueryWrapper<>();
            queryWrapper.lambda().eq(BasePowerLineEnt::getId, input.getId());
            BasePowerLineEnt entity = mapper.selectOne(queryWrapper);
            if (entity == null) {
                return XServiceResult.error(context, XError.NotFound);
            }
            XCopyUtils.copyObject(input, entity);
            mapper.updateById(entity);
            return XServiceResult.OK;
        });
    }

    @XApiAnonymous
    @XApiPost
    public XServiceResult deleteBasePowerLine(XContext context, DeleteBasePowerLineInput input) {
        BasePowerLineMapper mapper = context.getBean(BasePowerLineMapper.class);
        QueryWrapper<BasePowerLineEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(BasePowerLineEnt::getId, input.getId());
        BasePowerLineEnt entity = mapper.selectOne(queryWrapper);
        if (entity == null) {
            return XServiceResult.error(context, XError.NotFound);
        }
        mapper.deleteById(entity);
        return XServiceResult.OK;
    }

    @XApiAnonymous
    @XApiGet
    public XSingleResult<GetBasePowerLineOutput> getBasePowerLine(XContext context, GetBasePowerLineInput input) {
        BasePowerLineMapper mapper = context.getBean(BasePowerLineMapper.class);
        QueryWrapper<BasePowerLineEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(BasePowerLineEnt::getId, input.getId());
        BasePowerLineEnt entity = mapper.selectOne(queryWrapper);
        if (entity == null) {
            return XSingleResult.error(context, XError.NotFound);
        }
        GetBasePowerLineOutput output = new GetBasePowerLineOutput();
        XCopyUtils.copyObject(entity, output);
        return XSingleResult.success(output);
    }

    @XApiAnonymous
    @XApiGet
    public XSingleResult<GetBasePowerLineViewOutput> getBasePowerLineView(XContext context, GetBasePowerLineViewInput input) {
        BasePowerLineViewMapper mapper = context.getBean(BasePowerLineViewMapper.class);
        BasePowerLineView record = new BasePowerLineView();
        XCopyUtils.copyObject(input, record);
        BasePowerLineView view = mapper.selectOne(record);
        if (view == null) {
            return XSingleResult.error(context, XError.NotFound);
        }
        GetBasePowerLineViewOutput output = new GetBasePowerLineViewOutput();
        XCopyUtils.copyObject(view, output);
        return XSingleResult.success(output);
    }

    @XApiAnonymous
    @XApiGet
    public XPageResult<QueryBasePowerLineViewOutput> queryBasePowerLineView(XContext context, QueryBasePowerLineViewInput input) {
        BasePowerLineViewMapper mapper = context.getBean(BasePowerLineViewMapper.class);
        BasePowerLineView record = new BasePowerLineView();
        XCopyUtils.copyObject(input, record);
        PageHelper.startPage(input.getPage(), input.getLimit());
        List<BasePowerLineView> list = mapper.selectList(record);
        PageInfo<BasePowerLineView> pageInfo = new PageInfo<>(list);
        List<QueryBasePowerLineViewOutput> outputs = XCopyUtils.copyNewList(pageInfo.getList(), QueryBasePowerLineViewOutput.class);
        return XPageResult.success(outputs, input, pageInfo.getTotal());
    }

}
